#ifndef ScopeH
#define ScopeH

#include "..\miniscope\device\MiniscopeDeviceCapabilities.h"
#include <vector>
#include <string>
#include <windows.h>
#include "singleton.h"


class Scope : public CSingleton<Scope>
{
public:
	enum { CHANNELS = 4 };
	enum { SAMPLES_COUNT = 2048 };
	enum { SAMPLES_PER_CALLBACK_COUNT = 256 };
private:
	friend class CSingleton<Scope>;
	Scope();
	~Scope();

	bool bRunning;
	bool bStopRequest;

    unsigned int sample_count;  // counter for samples sent to user
	enum E_TRIGGER_MODE mode;
	bool pChannelEnable[CHANNELS];
    bool bManualTriggerRequest;
    unsigned int frame_position;	    ///< current data buffer position (0 when just sent trigger signal)
    HANDLE Thread;
    HANDLE hCommDev;
    DCB dcb;

    typedef void (*CallbackLog)(char*);
    typedef void (*CallbackData)(short*, int);
    typedef void (*CallbackConnect)(int, char*);
	typedef void (*CallbackTrigger)(void);
	typedef void (*CallbackUpdateTimebase)(int newIndex);
	typedef void (*CallbackUpdateSensitivity)(int channel, int newIndex);
	typedef void (*CallbackUpdateCoupling)(int channel, enum E_COUPLING_TYPE type);
	void log(const char* const lpData, ...);

	void OnDataReceive(const char* buf, unsigned int count);
	void OnLineReceive(const char* text);
	char line[256];
	unsigned int linePos;

	friend DWORD WINAPI ThreadProc(LPVOID data);
	bool exited;

    bool frameSamplesStarted;
	int frameSamplesCnt;
	int timebaseIndex;
	int sensitivityIndex;	// for analog channel 1 only
	float voltageMultiplier;
	struct ChannelData {
		short samples[SAMPLES_PER_CALLBACK_COUNT];
	} channelData[CHANNELS];

public:

    int Connect(void);
    int Disconnect(void);
	bool connected;

	static float fSampl[12];
	static float fSens[9];

    CallbackLog callbackLog;
    CallbackData callbackData;
    CallbackConnect callbackConnect;
	CallbackTrigger callbackTrigger;
	CallbackUpdateTimebase callbackUpdateTimebase;
	CallbackUpdateSensitivity callbackUpdateSensitivity;
	CallbackUpdateCoupling callbackUpdateCoupling;
    void ConnectCb(int state, char* msg)
    {
        if (callbackConnect)
            callbackConnect(state, msg);
    }
    /** \brief Request manual triggering
    	\return 0 on success
    */
    int ManualTrigger(void);
    int Run(bool state);
    int SetTriggerMode(enum E_TRIGGER_MODE);
    int EnableChannel(int channel, bool state);
    int SetTriggerType(enum E_TRIGGER_TYPE);
	int SetTriggerLevel(int level);

	void Handle(void);

	std::string comPort;
	
};

#endif
